@@ -483,6 +483,7 @@ g_process_set_caps(const gchar *caps)
 void
 g_process_set_argv_space(gint argc, gchar **argv)
 {
+#ifdef HAVE_ENVIRON
   extern char **environ;
   gchar *lastargv = NULL;
   gchar **envp    = environ;
@@ -526,6 +527,7 @@ g_process_set_argv_space(gint argc, gchar **argv)
   for (i = 0; envp[i] != NULL; i++)
     environ[i] = g_strdup(envp[i]);
   environ[i] = NULL;
+#endif
 }
 
 /**
@@ -1027,6 +1029,7 @@ g_process_perform_startup(void)
 static void
 g_process_setproctitle(const gchar* proc_title)
 {
+#ifdef HAVE_ENVIRON
   size_t len;
 
   g_assert(process_opts.argv_start != NULL);
@@ -1034,6 +1037,7 @@ g_process_setproctitle(const gchar* proc_title)
   len = g_strlcpy(process_opts.argv_start, proc_title, process_opts.argv_env_len);
   for (; len < process_opts.argv_env_len; ++len)
       process_opts.argv_start[len] = SPT_PADCHAR;
+#endif
 }
 
 
@@ -1436,6 +1440,14 @@ g_process_process_mode_arg(const gchar *option_name G_GNUC_UNUSED, const gchar *
   return TRUE;
 }
 
+static gboolean
+g_process_process_no_caps(const gchar *option_name G_GNUC_UNUSED, const gchar *value G_GNUC_UNUSED,
+                          gpointer data G_GNUC_UNUSED, GError *error)
+{
+  process_opts.caps = NULL;
+  return TRUE;
+}
+
 static GOptionEntry g_process_option_entries[] =
 {
   { "foreground",   'F', G_OPTION_FLAG_REVERSE, G_OPTION_ARG_NONE,     &process_opts.mode,              "Do not go into the background after initialization", NULL },
@@ -1446,7 +1458,7 @@ static GOptionEntry g_process_option_entries[] =
   { "gid",            0,  G_OPTION_FLAG_HIDDEN, G_OPTION_ARG_STRING,   &process_opts.group,             NULL, NULL },
   { "chroot",       'C',                     0, G_OPTION_ARG_STRING,   &process_opts.chroot_dir,        "Chroot to this directory", "<dir>" },
   { "caps",           0,                     0, G_OPTION_ARG_STRING,   &process_opts.caps,              "Set default capability set", "<capspec>" },
-  { "no-caps",        0, G_OPTION_FLAG_REVERSE, G_OPTION_ARG_NONE,     &process_opts.caps,              "Disable managing Linux capabilities", NULL },
+  { "no-caps",        0,  G_OPTION_FLAG_NO_ARG, G_OPTION_ARG_CALLBACK, g_process_process_no_caps,       "Disable managing Linux capabilities", NULL },
   { "pidfile",      'p',                     0, G_OPTION_ARG_STRING,   &process_opts.pidfile,           "Set path to pid file", "<pidfile>" },
   { "enable-core",    0,                     0, G_OPTION_ARG_NONE,     &process_opts.core,              "Enable dumping core files", NULL },
   { "fd-limit",       0,                  0, G_OPTION_ARG_INT,      &process_opts.fd_limit_min,         "The minimum required number of fds", NULL },
